@@ -1,7 +1,7 @@
 #' PLmixed: A package for estimating GLMMs with factor structures.
 #'
 #' The \code{PLmixed} package's main function is \code{\link{PLmixed}}, which estimates
-#' the model through nested maximizations using the package \pkg{\link{lme4}} package
+#' the model through nested maximizations using the \pkg{\link{lme4}} package
 #' and \code{\link{optim}} function. This extends the capabilities of \pkg{\link{lme4}}
 #' to allow for estimated factor structures, making it useful for estimating multilevel
 #' factor analysis and item response theory models with an arbitrary number of hierarchical
@@ -15,7 +15,7 @@ NULL
 #' Fit GLMM with Factor Structure
 #'
 #' Fit a (generalized) linear mixed effects model (GLMM) with factor structures. Utilizes both the
-#' \pkg{\link{lme4}} and \code{\link{optim}} packages for estimation using a profile-likelihood based
+#' \pkg{\link{lme4}} package and \code{\link{optim}} function for estimation using a profile-likelihood based
 #' approach.
 #' @param formula A formula following that of \pkg{\link{lme4}}, with the addition that factors can be specified
 #' as random effects. Factor names should not be names of variables in the data set, and are instead
@@ -32,10 +32,12 @@ NULL
 #' If a vector, the values apply in row by column by matrix order.
 #' @param nAGQ If family is non-gaussian, the number of points per axis for evaluating the adaptive
 #' Gauss-Hermite approximation to the log-likelihood. Defaults to \code{1}, corresponding to the Laplace approximation.
-#' See \pkg{\link{glmer}}.
+#' See \code{\link{glmer}}.
 #' @seealso \code{\link{glmer}}
 #' @seealso \code{\link{lmer}}
 #' @param method The \code{\link{optim}} optimization method. Defaults to \code{L-BFGS-B}.
+#' @param lower Lower bound on lambda parameters if \code{method = "L-BFGS-B"}.
+#' @param upper Upper bound on lambda parameters if \code{method = "L-BFGS-B"}.
 #' @param lme4.optimizer The \pkg{\link{lme4}} optimization method.
 #' @param lme4.start Start values used for \pkg{\link{lme4}}.
 #' @param lme4.optCtrl A list controlling the lme4 optimization. See \code{\link{lmerControl}}
@@ -85,7 +87,7 @@ NULL
 #' IRTsub <- IRTsub[order(IRTsub$item), ] # Order by item
 #' irt.lam = c(1, NA, NA) # Specify the lambda matrix
 #'
-#' # Below, the # in front of family = binomial can be used to change the response distribution
+#' # Below, the # in front of family = binomial can be removed to change the response distribution
 #' # to binomial, where the default link function is logit.
 #'
 #' irt.model <- PLmixed(y ~ 0 + as.factor(item) + (0 + abil.sid |sid) +(0 + abil.sid |school),
@@ -165,8 +167,9 @@ NULL
 ################## PLmixed Function ######################
 
 PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, factor = NULL, init = 1,
-                    nAGQ = 1, method = "L-BFGS-B", lme4.optimizer = "bobyqa", lme4.start = NULL, lme4.optCtrl = list(),
-                    opt.control = NULL, REML = FALSE, SE = 1, ND.method = "simple", est=TRUE) {
+                    nAGQ = 1, method = "L-BFGS-B", lower = -Inf, upper = Inf, lme4.optimizer = "bobyqa",
+                    lme4.start = NULL, lme4.optCtrl = list(),opt.control = NULL, REML = FALSE, SE = 1,
+                    ND.method = "simple", est=TRUE) {
 
   start.time <- proc.time()
   iter.counter.global <- 0
@@ -174,7 +177,7 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
   stop.iter.counter <- NULL
   model <- formula
 
-  # For Iteration SUmmary
+  # For Iteration Summary
   ll.list.global <- NULL
   lambda.est.global <- NULL
   fixed.effect.est.global <- NULL
@@ -184,21 +187,23 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
   new.lambda <- vector("list", length(load.var))
   new.factor <- vector("list", length(load.var))
 
-  if(is.null(lambda) == 1){
+  if(is.null(lambda)){
     lambda <- list(NA)
   }
 
   if(length(load.var) > length(lambda)){
-    for (i in ((length(lambda)+1):length(load.var))){
+    warning("More load.var listed than lambda matrices.",
+            immediate. = TRUE)
+    for (i in ((length(lambda) + 1):length(load.var))){
       lambda[i] = NA
     }
   }
 
-  if(is.null(factor) == 1){
+  if(is.null(factor)){
     factor <- list(NA)
   }
 
-  factors <- rep(NA,length(load.var))
+  factors <- rep(NA, length(load.var))
   col.vec <- rep(NA, length(load.var))
   uniq.list <- vector("list", length(load.var))
   num.vec <- rep(NA, length(load.var))
@@ -211,9 +216,9 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
 
     col <- which(colnames(data) == load.var[l])
     col.vec[l] <- col
-    uniq <- unique(data[,col])
+    uniq <- unique(data[, col])
     uniq.list[[l]] <- uniq
-    num <- length(unique(data[,col]))
+    num <- length(uniq)
     num.vec[l] <- num
 
 
@@ -228,41 +233,52 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
     # values in load.var.
     # For each factor, one value in Lambda is constrained
     # to 1.
-    if(is.na(lambda[l]) == 1){
+
+    # The exception is if the load.var contains only 1 unique element.
+    # This allows for constraints and nonlinear fixed effect parameters
+    # to be included in the model using a single element lambda matrix
+    # corresponding to a load.var with 1 unique element and a nonlinear parameter
+    # name given in factor, which is included in the model formula.
+
+    if(is.na(lambda[l]) & num > 1){
       tmp.lambda <- NULL
     }
     else{
       tmp.lambda <- lambda[[l]]
     }
 
-    if(is.na(factor[l]) == 1){
+    if(is.na(factor[l])){
       tmp.factor <- NULL
     }
     else{
       tmp.factor <- factor[[l]]
     }
 
-    if(is.null(tmp.lambda) == 1){
-      if(is.null(tmp.factor) == 1){
+    if(is.null(tmp.lambda)){
+      if(is.null(tmp.factor)){
+        warning("For load.var = ", paste(load.var[l]), "\nNo lambda and factor arguments specified.\n",
+                "Lambda will be a 1 x q matrix where q = length(unique(load.var)).\n",
+                "lambda[1,1] is constrained to 1.",
+                immediate. = TRUE)
         tmp.lambda <- matrix(NA, nrow = num, ncol = 1)
-        tmp.lambda[1,1] <- 1
+        tmp.lambda[1, 1] <- 1
       }
       else{
+        warning("For load.var = ", paste(load.var[l]), "\nNo lambda matrix provided.\nLambda will be ",
+                "an f x q matrix where f is the number of factor names provided and q = length(unique(load.var)).\n",
+                "One element of each column in lambda is constrained to 1.",
+                immediate. = TRUE)
         tmp.lambda <- matrix(NA, nrow = num, ncol = length(tmp.factor))
         for(i in 1:length(tmp.factor)){
-          tmp.lambda[i,i] <- 1
+          tmp.lambda[i, i] <- 1
         }
       }
     }
 
 
-    # Determine how many parameters in Lambda must be estimated.
-    # For those that must be estimated, start values are assigned
-    # based on those provided by the user. If none are provided,
-    # the start values used are 1.
 
     num.est.par <- sum(is.na(tmp.lambda))
-    tot.est <- c(tot.est,num.est.par)
+    tot.est <- c(tot.est, num.est.par)
 
 
     # Determines the number of factors specified. If factor
@@ -270,12 +286,15 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
     # the number of names provided.
     # If no names are provided, the number of factors is set to the
     # number of columns in Lambda. Factor names are assigned column
-    # by column as f1, f2, etc.
+    # by column from lambda to lambda as f1.1, f2.1, f1.2, f2.2, etc.
 
-    if (is.null(tmp.factor) == 0){
+    if (!is.null(tmp.factor)){
       factors[l] <- length(tmp.factor)
     }
     else{
+      warning("For load.var = ", paste(load.var[l]), "\nNo factor names provided.\n",
+              "Factors named as fc.v, where c is the column number of the vth load.var.",
+              immediate. = TRUE)
       factors[l] <- ncol(as.matrix(tmp.lambda))
       tmp.factor <- paste("f", 1:factors[l],".",l, sep="")
     }
@@ -286,6 +305,11 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
 
   }
 
+  # Determine how many parameters in Lambda must be estimated.
+  # For those that must be estimated, start values are assigned
+  # based on those provided by the user. If none are provided,
+  # the start values used are 1.
+
   tot.est <- sum(tot.est)
 
   if (length(init) == 1){
@@ -297,29 +321,31 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
 
   lambda <- new.lambda
   factor <- new.factor
-  fam.string <- deparse(substitute(family))
-  fam <- substitute(family)
 
-
-  #print(lambda)
-  #print(factor)
-  #print(init.tot)
+  # Prepare family argument for fitting with lme4
+  # Taken from glmer code to format the family
+  # argument correctly. Will be used to determine if
+  # lmer or glmer is used.
+  if (is.character(family)){
+    family <- get(family, mode = "function", envir = parent.frame(2))
+  }
+  if( is.function(family)){
+    family <- family()
+  }
 
   ################ La.load Function ########################
 
   La.load <- function(start, model, data, load.var, factor = NULL, consts = NULL,
-                      est=F, lik=T, delta=F, k=1, derivs = T, nAGQ = 1) {
+                      est = FALSE, lik = TRUE, delta = FALSE, k = 1, derivs = TRUE, nAGQ = 1) {
 
     obs <- data
-
     time.local <- proc.time()
     ### num.est is used to fill in the NAs from the start values in the next loop
     num.est <- 1
 
-    if (is.null(stop.iter.counter) == 1){
-
+    if (is.null(stop.iter.counter)){
       assign('iter.counter.global', iter.counter.global + 1, inherits = TRUE)
-      cat('\r',paste0('Iteration Number: ',iter.counter.global))
+      cat('\r',paste0('Iteration Number: ', iter.counter.global))
     }
 
 
@@ -338,11 +364,11 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
 
         ### Apply contstraints and lambda values to weighted.var
         for(i in 1:num){
-          if(is.na(consts.2[i,q]) == 0){
-            obs$weighted.var[obs[,col]==uniq[i]] <- consts.2[i,q]
+          if(is.na(consts.2[i, q]) == 0){
+            obs$weighted.var[obs[, col] == uniq[i]] <- consts.2[i, q]
           }
           else {
-            obs$weighted.var[obs[,col]==uniq[i]] <- start[num.est]
+            obs$weighted.var[obs[, col] == uniq[i]] <- start[num.est]
             num.est <- num.est + 1
           }
         }
@@ -363,22 +389,33 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
     }
 
 
-    ### Estimate GLMM
-    if (fam.string == "gaussian") {
-      lmer.result <- lme4::lmer(model, data=obs, REML = REML, start = lme4.start.val,
-                                control = lmerControl(calc.derivs = derivs, optimizer = lme4.optimizer, optCtrl = lme4.optCtrl))
-      assign('lme4.initial.theta.values', lme4::getME(lmer.result, name = "theta"), inherits = TRUE)
+    ### Estimate GLMM - Use lmer if family = gaussian(link = "identity") and glmer for others
+    if (isTRUE(all.equal(family, gaussian()))) {
+      lmer.result <- lme4::lmer(model, data = obs, REML = REML, start = lme4.start.val,
+                                control = lmerControl(calc.derivs = derivs,
+                                                      optimizer = lme4.optimizer,
+                                                      optCtrl = lme4.optCtrl))
+
+      # Obtain estimates to use as lme4 starting values for next optim iteration
+      assign('lme4.initial.theta.values', lme4::getME(lmer.result, name = "theta"),
+             inherits = TRUE)
     }
     else {
-      lmer.result <- lme4::glmer(model, data=obs, family=eval(fam), start = lme4.start.val,
-                                 control = glmerControl(calc.derivs = derivs, optimizer = lme4.optimizer, optCtrl = lme4.optCtrl),
-                                 nAGQ = nAGQ)
+      lmer.result <- lme4::glmer(model, data = obs, family = family, start = lme4.start.val,
+                                 control = glmerControl(calc.derivs = derivs,
+                                                        optimizer = lme4.optimizer,
+                                                        optCtrl = lme4.optCtrl),
+                                                        nAGQ = nAGQ)
+
+      # Obtain estimates to use as lme4 starting values for next optim iteration
       if (nAGQ == 0){
-        assign('lme4.initial.theta.values',lme4::getME(lmer.result, name = "theta"), inherits = TRUE)
+        assign('lme4.initial.theta.values',lme4::getME(lmer.result, name = "theta"),
+               inherits = TRUE)
       }
       else{
         assign('lme4.initial.theta.values', list(fixef = lme4::fixef(lmer.result),
-                                                 theta=lme4::getME(lmer.result, name = "theta")), inherits = TRUE)
+                                                 theta=lme4::getME(lmer.result, name = "theta")),
+               inherits = TRUE)
       }
     }
 
@@ -390,23 +427,25 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
     #print(summary(lmer.result))
 
     time.local.2 <- proc.time()
-    time.dif <- time.local.2-time.local
+    time.dif <- time.local.2 - time.local
 
     assign('ll.list.global', rbind(ll.list.global,ll), inherits = TRUE)
     assign('lambda.est.global', rbind(lambda.est.global,start), inherits = TRUE)
-    assign('fixed.effect.est.global', rbind(fixed.effect.est.global,lme4::fixef(lmer.result)), inherits = TRUE)
-    assign('random.effect.est.global', rbind(random.effect.est.global,lme4::getME(lmer.result, name = "theta")), inherits = TRUE)
+    assign('fixed.effect.est.global', rbind(fixed.effect.est.global,lme4::fixef(lmer.result)),
+           inherits = TRUE)
+    assign('random.effect.est.global', rbind(random.effect.est.global,
+                                             lme4::getME(lmer.result, name = "theta")), inherits = TRUE)
     assign('time.global', rbind(time.global,(time.dif[3])), inherits = TRUE)
 
-    if(lik==TRUE){
+    if(lik == TRUE){
       return(-ll)
     }
 
-    if(est==TRUE){
+    if(est == TRUE){
       return(list(ll=-ll, total=lmer.result, degfree = df.residual(lmer.result)))
     }
 
-    if(delta==TRUE){
+    if(delta == TRUE){
       fix.b <- fixef(lmer.result)[k]
       return(fix.b)
     }
@@ -415,9 +454,9 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
 
   ########## End La.Load Function ##########
 
-  opt.result <- optim(par = init.tot, fn = La.load, hessian=T, method = method, control = opt.control,
-                      consts = lambda, factor = factor, data = data, load.var = load.var,
-                      model = model, est=F, lik=T, delta=F, derivs = F, nAGQ = nAGQ)
+  opt.result <- optim(par = init.tot, fn = La.load, hessian = TRUE, method = method, control = opt.control,
+                      consts = lambda, factor = factor, data = data, load.var = load.var, lower = lower, upper = upper,
+                      model = model, est = FALSE, lik = TRUE, delta = FALSE, derivs = FALSE, nAGQ = nAGQ)
 
 
   # value 1
@@ -433,7 +472,7 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
   }
 
 
-  final.lik <- (opt.result$value)*(-1)
+  final.lik <- (opt.result$value) * (-1)
   tot.est.par <- length(Est)
   total.iters <- iter.counter.global
 
@@ -443,9 +482,10 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
   assign('stop.iter.counter', 1, inherits = TRUE)
 
   # value 4
-  if (est==TRUE) {
+  if (est == TRUE) {
     final.model <- La.load(start = opt.result$par, consts = lambda, data = data,
-                           factor = factor, load.var = load.var, model = model, est=T, lik=F, nAGQ = nAGQ)
+                           factor = factor, load.var = load.var, model = model,
+                           est = TRUE, lik = FALSE, nAGQ = nAGQ)
 
     final <- final.model$total
 
@@ -461,25 +501,23 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
       #Direct numerical derivative
       eps <- 10^(-4)
       iden <- diag(1, nrow = tot.est.par, ncol = tot.est.par)
-      beta.ep <- matrix(0,num.fix.ef, tot.est.par)
+      beta.ep <- matrix(0, num.fix.ef, tot.est.par)
 
       rep.lam <- vector("list", length(lambda))
       for(i in 1:tot.est.par){
-        lam.new <- Est + iden[,i]*eps
-
+        lam.new <- Est + iden[, i]*eps
         cycle <- 1
-
         for(c in 1:length(lambda)){
           n.lam <- lambda[[c]]
           full.lam <- matrix(0,nrow = nrow(n.lam), ncol = ncol(n.lam))
 
           for(p in 1:ncol(n.lam)){
             for(q in 1:nrow(n.lam)){
-              if(is.na(n.lam[q,p]) == 0){
-                full.lam[q,p] <- n.lam[q,p]
+              if(is.na(n.lam[q, p]) == 0){
+                full.lam[q, p] <- n.lam[q, p]
               }
               else{
-                full.lam[q,p] <- lam.new[cycle]
+                full.lam[q, p] <- lam.new[cycle]
                 cycle <- cycle + 1
               }
             }
@@ -489,16 +527,16 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
 
         final.new <- La.load(start = lam.new, consts = rep.lam, data = data,
                              factor = factor, load.var = load.var, model = model,
-                             est=T, lik=F, delta=F, nAGQ = nAGQ)
+                             est = TRUE, lik = FALSE, delta = FALSE, nAGQ = nAGQ)
         beta.ep[,i] <- lme4::fixef(final.new$total)
       }
 
-      grad.b <- (matrix(rep(beta,tot.est.par),num.fix.ef,tot.est.par,byrow=F) - beta.ep)/ eps
+      grad.b <- (matrix(rep(beta,tot.est.par), num.fix.ef,tot.est.par, byrow = FALSE) - beta.ep)/ eps
 
       #Adjusted standard errors for beta
       adj.se.b <- rep(0, num.fix.ef)
       for(i in 1:num.fix.ef){
-        adj.se.b[i] <- sqrt(naive[i] + t(grad.b[i,]) %*% cov.mat %*% grad.b[i,])
+        adj.se.b[i] <- sqrt(naive[i] + t(grad.b[i, ]) %*% cov.mat %*% grad.b[i, ])
       }
 
     }
@@ -510,15 +548,15 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
 
       grad.b2 <- matrix(0, num.fix.ef, tot.est.par)
       for(i in 1:num.fix.ef){
-        grad.b2[i,] <- numDeriv::grad(La.load, method= ND.method, x = Est, consts = lambda, data = data,
-                                      factor = factor, load.var = load.var, model = model, lik=F,
-                                      est=F, delta=T,k=i, nAGQ = nAGQ)
+        grad.b2[i, ] <- numDeriv::grad(La.load, method= ND.method, x = Est, consts = lambda, data = data,
+                                      factor = factor, load.var = load.var, model = model, lik = FALSE,
+                                      est = FALSE, delta = TRUE, k = i, nAGQ = nAGQ)
       }
 
       # Adjust standard errors
       adj.se.b <- rep(0,num.fix.ef)
       for (i in 1:num.fix.ef){
-        adj.se.b[i] <- sqrt(naive[i] + t(grad.b2[i,]) %*% cov.mat %*% grad.b2[i,])
+        adj.se.b[i] <- sqrt(naive[i] + t(grad.b2[i, ]) %*% cov.mat %*% grad.b2[i, ])
       }
 
     }
@@ -536,7 +574,8 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
       colnames(fix) <- c("Beta", "SE", "t value")
     }
     else{
-      fix <- cbind(beta, adj.se.b, beta/adj.se.b, 2*pnorm(abs(beta/adj.se.b), lower.tail = F))
+      fix <- cbind(beta, adj.se.b, beta/adj.se.b,
+                   2*pnorm(abs(beta/adj.se.b), lower.tail = F))
       colnames(fix) <- c("Beta", "SE", "z value", "Pr(>|z|)")
     }
     rand.ef <- lme4::VarCorr(final)
@@ -554,23 +593,23 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
       factor.2 <- factor[[h]]
       lambda.1 <- lambda[[h]]
 
-      final.lambda <- matrix(NA, nrow = num.vec[h], ncol = 2*(length(factor.2)))
+      final.lambda <- matrix(NA, nrow = num.vec[h], ncol = 2 * (length(factor.2)))
       se.names <- rep("SE", length(factor.2))
       comb <- rbind(factor.2, se.names)
-      comb <- matrix(comb, nrow = 1, ncol = 2*length(factor.2), byrow = F)
+      comb <- matrix(comb, nrow = 1, ncol = 2 * length(factor.2), byrow = F)
       rownames(final.lambda) <- uniq
       colnames(final.lambda) <- comb
 
 
       for(i in 1:nrow(as.matrix(lambda.1))){
         for (j in 1:ncol(as.matrix(lambda.1))){
-          if(is.na(lambda.1[i,j])==1){
-            final.lambda[i,(j*2-1)] <- Est[fill]
-            final.lambda[i,j*2] <- st.er[fill]
-            fill <- fill+1
+          if(is.na(lambda.1[i, j]) == 1){
+            final.lambda[i, (j * 2 - 1)] <- Est[fill]
+            final.lambda[i, j * 2] <- st.er[fill]
+            fill <- fill + 1
           }
           else{
-            final.lambda[i,(j*2-1)] <- lambda.1[i,j]
+            final.lambda[i, (j * 2 - 1)] <- lambda.1[i, j]
           }
         }
       }
@@ -595,9 +634,9 @@ PLmixed <- function(formula, data, family = gaussian, load.var, lambda = NULL, f
                          "Fixed Effects" = fixed.effect.est.global, "Random Effects" = random.effect.est.global,
                          "Time" = time.global)
 
-    final.object <- list("Log-Likelihood" = final.lik, "Fixed Effects"=fix, "Random Effects"=rand.ef,
-                         "Lambda"=fin.lam, "Cov Matrix"=cov.mat, "Error code"=code, "Total Iterations"=total.iters,
-                         "lme4 Model"=final, "Iteration Summary" = iter.summary, "Model" = model, "Family" = fam.string,
+    final.object <- list("Log-Likelihood" = final.lik, "Fixed Effects" = fix, "Random Effects" = rand.ef,
+                         "Lambda" = fin.lam, "Cov Matrix" = cov.mat, "Error code" = code, "Total Iterations" = total.iters,
+                         "lme4 Model"=final, "Iteration Summary" = iter.summary, "Model" = model, "Family" = family(final),
                          "Data" = deparse(substitute(data)), "Load.Var" = load.var, "Factor" = factor, "nAGQ" = nAGQ,
                          "Lambda.raw" = new.lambda, "Param" = Est, "Estimation Time" = total.estimation.time, "REML" = reml,
                          "Optimizer" = optimizers)
